<soapenv:Envelope xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"  xmlns:py="http://genshi.edgewall.org/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
py:attrs="{'xmlns' : typenamespace}">
    <soapenv:Body>
        ${soap_body(methodname, function_info, output)}
    </soapenv:Body>
</soapenv:Envelope>
